@@ -8,17 +8,19 @@ package Petal;
 use Petal::Hash;
 use Petal::Cache::Disk;
 use Petal::Cache::Memory;
-use Petal::Parser::XMLWrapper;
-use Petal::Parser::HTMLWrapper;
+use Petal::Parser;
 use Petal::Canonicalizer::XML;
 use Petal::Canonicalizer::XHTML;
 use Petal::Functions;
+use Petal::Entities;
 use File::Spec;
-use strict;
-use warnings;
 use Carp;
 use Safe;
 use Data::Dumper;
+use Encode;
+use MKDoc::XML::DecodeHO;
+use strict;
+use warnings;
 
 
 # these are used as local variables when the XML::Parser
@@ -42,9 +44,9 @@ our $WARN_UNINIT = 0;
 # What do we use to parse input?
 our $INPUT  = 'XML';
 our $INPUTS = {
-    'XML'   => 'Petal::Parser::XMLWrapper',
-    'HTML'  => 'Petal::Parser::HTMLWrapper',
-    'XHTML' => 'Petal::Parser::HTMLWrapper',
+    'XML'     => 'Petal::Parser',
+    'HTML'    => 'Petal::Parser',
+    'XHTML'   => 'Petal::Parser',
 };
 
 
@@ -84,7 +86,7 @@ our $CURRENT_INCLUDES = 0;
 
 
 # this is for CPAN
-our $VERSION = '1.00';
+our $VERSION = '1.10_01';
 
 
 # The CodeGenerator class backend to use.
@@ -144,14 +146,15 @@ sub main::lcode
 
 sub load_code_generator
 {
-	if (not $CodeGeneratorLoaded)
-	{
-	    eval "require $CodeGenerator";
-	    confess "Failed to load $CodeGenerator, $@" if $@;
-	    $CodeGeneratorLoaded = 1;
-	}
+    if (not $CodeGeneratorLoaded)
+    {
+	eval "require $CodeGenerator";
+	confess "Failed to load $CodeGenerator, $@" if $@;
+	$CodeGeneratorLoaded = 1;
+    }
 }
 
+
 # Instanciates a new Petal object.
 sub new
 {
@@ -229,7 +232,6 @@ sub _base_dir
 }
 
 
-
 # _include_compute_path ($path);
 # ------------------------------
 # Computes the new absolute path from the current
@@ -285,6 +287,28 @@ sub process
 {
     my $self = shift;
     
+    # sets $BASE_DIR, @BASE_DIR and $self->{base_dir} to absolute paths once
+    # because File::Spec::rel2abs() can be quite expensive
+    $BASE_DIR = File::Spec->rel2abs ($BASE_DIR) if (defined $BASE_DIR and substr ($BASE_DIR, 0, 1) ne '/');
+    
+    @BASE_DIR = ( map { substr ($_, 0, 1) ne '/' ? File::Spec->rel2abs ($_) : $_ }
+		  map { defined $_ ? $_ : () } @BASE_DIR );
+
+    if ($self->{base_dir})
+    {
+	if (ref $self->{base_dir})
+	{
+	    $self->{base_dir} = [
+		map { substr ($_, 0, 1) ne '/' ? File::Spec->rel2abs ($_) : $_ }
+		map { defined $_ ? $_ : () } @{$self->{base_dir}}
+	       ] if (defined $self->{base_dir});
+	}
+	else
+	{
+	    $self->{base_dir} = File::Spec->rel2abs ($self->{base_dir}) unless (substr ($self->{base_dir}, 0, 1) ne '/');
+	}
+    }
+    
     # ok, from there on we need to override any global
     # variable with stuff that might have been specified
     # when constructing the object
@@ -318,7 +342,6 @@ sub process
 	$res = $coderef->($hash);
 	
 	$Petal::ENCODE_CHARSET and do {
-	    require "Encode.pm";
 	    $res = Encode::encode ($Petal::ENCODE_CHARSET, $res);
 	};
     };
@@ -344,7 +367,7 @@ sub _handle_error
 	ref $error and do {
 	    print ERROR "=============\n";
 	};
-	print "\n";
+	print ERROR "\n";
 	
 	print ERROR "Petal object dump:\n";
 	print ERROR "==================\n";
@@ -424,9 +447,10 @@ sub _file_path
     
     foreach my $dir (@dirs)
     {
-	my $base_dir = File::Spec->canonpath ($dir);
-	$base_dir = File::Spec->rel2abs ($base_dir) unless ($base_dir =~ /^\//);
-	$base_dir =~ s/\/$//;
+	# my $base_dir  = File::Spec->canonpath ($dir);
+	# $base_dir     = File::Spec->rel2abs ($base_dir) unless ($base_dir =~ /^\//);
+	my $base_dir  = $dir;
+	$base_dir     =~ s/\/$//;
 	my $file_path = File::Spec->canonpath ($base_dir . '/' . $file);
 	return $file_path if (-e $file_path and -r $file_path);
     }
@@ -437,8 +461,8 @@ sub _file_path
 
 # $self->_file_data_ref;
 # ----------------------
-#   slurps the template data into a variable and returns a
-#   reference to that variable
+# slurps the template data into a variable and returns a
+# reference to that variable
 sub _file_data_ref
 {
     my $self      = shift;
@@ -451,10 +475,16 @@ sub _file_data_ref
     no bytes;
     
     $Petal::DECODE_CHARSET and do {
-	require "Encode.pm";
 	$res = Encode::decode ($Petal::DECODE_CHARSET, $res);
     };
     
+    if ($OUTPUT eq 'HTML' or $OUTPUT eq 'XHTML')
+    {
+	Encode::_utf8_on ($res);
+	$res = MKDoc::XML::DecodeHO->process ($res);
+	Encode::_utf8_off ($res);
+    }
+    
     # kill template comments
     $res =~ s/\<!--\?.*?\-->//gsm;
     return \$res;
@@ -505,9 +535,8 @@ sub _code_memory_cached
 	    $cpt->permit ('entereval');
 	    $cpt->permit ('leaveeval');
 	    $cpt->permit ('require');
-	    
 	    $cpt->reval($code_perl);
-	    die $@ if ($@);
+	    confess ($@ . "\n" . $self->_code_with_line_numbers) if $@;
 	    
 	    # remove silly warning '"Petal::CPT::VAR1" used only once'
 	    $Petal::CPT::VAR1 if (0);
@@ -516,7 +545,7 @@ sub _code_memory_cached
 	else
 	{
 	    eval "$code_perl";
-	    confess $@ if (defined $@ and $@);
+	    confess ($@ . "\n" . $self->_code_with_line_numbers) if $@;
 	    $code = $VAR1;
 	}
 	
@@ -553,6 +582,15 @@ sub _canonicalize
 }
 
 
+sub _utf8_on
+{
+    my $class = shift;
+    my $res   = shift;
+    Encode::_utf8_on ($res);
+    return $res;
+}
+
+
 1;
 
 =head1 NAME
@@ -1463,12 +1501,13 @@ C<user> variable which would be outside the loop. The template engine also
 provides the following variables for you inside the loop:
 
   <?repeat name="foo bar"?>
-    <?var name="__count__"?>    - iteration number, starting at 1
-    <?var name="__is_first__"?> - is it the first iteration?
-    <?var name="__is_last__"?>  - is it the last iteration?
-    <?var name="__is_inner__"?> - is it not the first and not the last iteration?
-    <?var name="__even__ "?>    - is the count even?
-    <?var name="__odd__"?>      - is the count odd?
+    <?var name="repeat/index"?>  - iteration number, starting at 0
+    <?var name="repeat/number"?> - iteration number, starting at 1
+    <?var name="repeat/start"?>  - is it the first iteration?
+    <?var name="repeat/end"?>    - is it the last iteration?
+    <?var name="repeat/inner"?>  - is it not the first and not the last iteration?
+    <?var name="repeat/even"?>   - is the count even?
+    <?var name="repeat/odd"?>    - is the count odd?
   <?end?>
 
 Again these variables are scoped, you can safely nest loops, ifs etc...  as
@@ -1709,7 +1748,11 @@ XML::Parser is deprecated and should be replaced by SAX handlers at some point.
 
 =head1 AUTHOR
 
-Copyright 2002 - Jean-Michel Hiver <jhiver@mkdoc.com> 
+Copyright 2003 - MKDoc Holdings Ltd.
+
+Authors: Jean-Michel Hiver <jhiver@mkdoc.com>, 
+         Fergal Daly <fergal@esatclear.ie>,
+	 and others.
 
 This module free software and is distributed under the same license as Perl
 itself. Use it at your own risk.
