@@ -10,6 +10,7 @@ use strict;
 use warnings;
 use File::Spec;
 use Digest::MD5 qw /md5_hex/;
+use Encode;
 use Carp;
 
 
@@ -56,10 +57,14 @@ sub set
     my $data  = shift;
     my $key   = $class->compute_key ($file);
     my $tmp   = $class->tmp;
-    open FP, ">$tmp/$key" or
+    {
+	no utf8;
+	use bytes;
+	open FP, ">$tmp/$key" or
         ( Carp::cluck "Cannot write-open $tmp/$key" and return );
-    print FP $data;
-    close FP;
+	print FP $data;
+	close FP;
+    }
 }
 
 
@@ -138,13 +143,21 @@ sub cached
     my $cached_filepath = $tmp . '/' . $key;
     
     (-e $cached_filepath) or return;
+
+    my $res = undef;
+    {
+	no utf8;
+	use bytes;
+	open FP, "<$tmp/$key" or
+            (Carp::cluck "Cannot read-open cached file for $tmp/$key" and return);
+	$res = join '', <FP>;
+	close FP;
+    }
+    $Petal::DECODE_CHARSET and do {
+	$res = Encode::decode ($Petal::DECODE_CHARSET, $res);
+    };
     
-    open FP, "<$tmp/$key" or
-        (Carp::cluck "Cannot read-open cached file for $tmp/$key" and return);
-    my $data = join '', <FP>;
-    close FP;
-    
-    return $data;
+    return $res;
 }
 
 
