@@ -7,7 +7,7 @@
 # code that can be turned into a subroutine using eval().
 # ------------------------------------------------------------------
 package Petal::CodeGenerator;
-use Petal::XML_Encode_Decode;
+use MKDoc::XML::Decode;
 use strict;
 use warnings;
 use Carp;
@@ -138,7 +138,7 @@ sub process
 	    %token_hash = (@atts1, @atts2); 
 	    foreach my $key (%token_hash)
 	    {
-		$token_hash{$key} = Petal::XML_Encode_Decode::decode_backslash_semicolon ($token_hash{$key})
+		$token_hash{$key} = $class->_decode_backslash_semicolon ($token_hash{$key})
 		    if (defined $token_hash{$key});
 	    }
 	    
@@ -202,16 +202,12 @@ sub _include
         $class->add_code ("my \$res = eval { Petal->new (file => '$path', lang => '$lang')->process (\$hash->new()) };") :
 	$class->add_code ("my \$res = eval { Petal->new ('$path')->process (\$hash->new()) };");
     
-    $class->add_code ("\$res = \$\@ if (defined \$\@ and \$\@);");
-    $class->add_code ("if (scalar keys \%Encode::) {");
-    $class->indent_increment();
+    $class->add_code ("\$res = \"<!--\\n\$\@\\n-->\" if (defined \$\@ and \$\@);");
     $class->add_code ("\$res = Encode::decode (\$Petal::ENCODE_CHARSET, \$res) if (\$Petal::ENCODE_CHARSET);");
-    $class->add_code ("Encode::_utf8_on (\$res) if (Encode->can ('_utf8_on'));");
-    $class->indent_decrement();
-    $class->add_code ("}");
+    $class->add_code ("\$res = Petal->_utf8_on (\$res);");
     $class->add_code ("\$res;");
     $class->indent_decrement();
-    $class->add_code ("};");
+    $class->add_code ("} || '';");
 }
 
 
@@ -226,14 +222,19 @@ sub _var
     
     (defined $variable and $variable) or
         confess "Cannot parse $token : 'name' attribute is not defined";
-
+    
     # set the variable in the $variables hash
     my $tmp = $variable;
     $tmp =~ s/\..*//;
     $variables->{$tmp} = 1;
     
     $variable =~ s/\'/\\\'/g;
-    $class->add_code($class->_add_res($class->comp_expr($variable).";"));
+    $class->add_code ( $class->_add_res (('do {')) );
+    $class->indent_increment();
+    $class->add_code ('my $res = ' . $class->comp_expr ($variable) . ';');
+    $class->add_code ('(defined $res) ? $res : "";');
+    $class->indent_decrement();
+    $class->add_code ('};');
 }
 
 
@@ -285,8 +286,8 @@ sub _endeval
     
     $class->add_code("return " . $class->_get_res() . ";");
     $class->indent_decrement();
-    $class->add_code("};");
-
+    $class->add_code("} || '';");
+    
     $class->add_code("if (defined \$\@ and \$\@) {");
     $class->indent_increment();
     $variable = quotemeta ($variable);
@@ -304,7 +305,7 @@ sub _attr
     my $class = shift;
     my $attribute = $token_hash{name} or
         confess "Cannot parse $token : 'name' attribute is not defined";
-
+    
     my $variable = $token_hash{value} or
         confess "Cannot parse $token : 'value' attribute is not defined";
     
@@ -317,11 +318,17 @@ sub _attr
     $variables->{$tmp} = 1;
     
     $variable =~ s/\'/\\\'/g;
-    $class->add_code("if (defined ".$class->comp_expr($variable)." and ".$class->comp_expr($variable)." ne '') {");
+    $class->add_code('{');
     $class->indent_increment();
-    $class->add_code($class->_add_res("\"$attribute\" . '=\"' . ".$class->comp_expr($variable)." . '\"'"));
+    $class->add_code ("my \$value = " . $class->comp_expr($variable) . ";");
+    $class->add_code ("if (defined(\$value)) {");
+    # $class->add_code ("if (defined(\$value) and length(\$value)) {");
+    $class->indent_increment();
+    $class->add_code ($class->_add_res (qq {"$attribute=\\"\$value\\""}) );
     $class->indent_decrement();
-    $class->add_code("}");
+    $class->add_code ("}");
+    $class->indent_decrement();
+    $class->add_code ("}");
 }
 
 
@@ -373,21 +380,42 @@ sub _for
     {
 	$class->add_code("\@array = \@{".$class->comp_expr($variable)."};");
     }
-    
-    $class->add_code("for (my \$i=0; \$i < \@array; \$i++) {");
+
+        
+    $class->add_code ("for (my \$i=0; \$i < \@array; \$i++) {");
     $class->indent_increment();
-    $class->add_code("my \$hash = \$hash->new();");
-    $class->add_code("my \$count= \$i + 1;");
-    $class->add_code("\$hash->{__count__}    = \$count;");
-    $class->add_code("\$hash->{__is_first__} = (\$count == 1);");
-    $class->add_code("\$hash->{__is_last__}  = (\$count == \@array);");
-    $class->add_code("\$hash->{__is_inner__} = " .
-		                        "(not \$hash->{__is_first__} " . 
-		                        "and not \$hash->{__is_last__});");
+    $class->add_code ("my \$hash   = \$hash->new();");
     
-    $class->add_code("\$hash->{__even__}     = (\$count % 2 == 0);");
-    $class->add_code("\$hash->{__odd__}      = not \$hash->{__even__};");
-    $class->add_code("\$hash->{'$as'} = \$array[\$i];");
+    # compute various might-be-useful variables
+    $class->add_code ("my \$number = \$i + 1;");
+    $class->add_code ("my \$odd    = \$number % 2;");
+    $class->add_code ("my \$even   = \$i % 2;");
+    $class->add_code ("my \$start  = (\$i == 0);");
+    $class->add_code ("my \$end    = (\$i == \@array);");
+    $class->add_code ("my \$inner  = (\$i and \$i < \@array);");
+    
+    # backwards compatibility
+    $class->add_code ("\$hash->{__count__}    = \$number;");
+    $class->add_code ("\$hash->{__is_first__} = \$start;");
+    $class->add_code ("\$hash->{__is_last__}  = \$end;");
+    $class->add_code ("\$hash->{__is_inner__} = \$inner;");
+    $class->add_code ("\$hash->{__even__}     = \$even;");
+    $class->add_code ("\$hash->{__odd__}      = \$odd;");
+    
+    # new repeat style object
+    $class->add_code ("\$hash->{repeat} = {");
+    $class->indent_increment();
+    $class->add_code ("index  => \$i,");
+    $class->add_code ("number => \$number,");
+    $class->add_code ("even   => \$even,");
+    $class->add_code ("odd    => \$odd,");
+    $class->add_code ("start  => \$start,");
+    $class->add_code ("end    => \$end,");
+    $class->add_code ("inner  => \$inner,");
+    $class->add_code ("};");
+    $class->indent_decrement();
+    
+    $class->add_code ("\$hash->{'$as'} = \$array[\$i];");
 }
 
 
@@ -414,4 +442,13 @@ sub _tokenize
 }
 
 
+sub _decode_backslash_semicolon
+{
+    my $class = shift;
+    my $data  = shift;
+    $data =~ s/&($MKDoc::XML::Decode::XML_Decode_Pattern)\\;/$MKDoc::XML::Decode::XML_Decode{$1}/go;
+    return $data;
+}
+
+
 1;
