@@ -1,15 +1,8 @@
-# ------------------------------------------------------------------
-# Petal::Hash - Context for petal objects
-# ------------------------------------------------------------------
-# Author: Jean-Michel Hiver
-# Description: This class manages the context objects, which holds
-# objects which are accessable from the template.
-# ------------------------------------------------------------------
 package Petal::Hash;
+use MKDoc::XML::Encode;
 use strict;
 use warnings;
 use Carp;
-use Petal::XML_Encode_Decode;
 
 our $MODIFIERS = {};
 
@@ -120,6 +113,17 @@ sub get
 }
 
 
+sub delete_cached
+{
+    my $self  = shift;
+    my $regex = shift;
+    for (keys %{$self->{__petal_hash_cache__}})
+    {
+	/$regex/ and delete $self->{__petal_hash_cache__}->{$_};
+    }
+}
+
+
 sub __FETCH
 {
     my $self = shift;
@@ -135,22 +139,13 @@ sub __FETCH
 	my $res = $self->fetch ($key);
 	if (defined $res and not ref $res)
 	{
-	    $res = $self->_xml_encode ($res);
+	    $res = MKDoc::XML::Encode->process ($res);
 	}
 	return $res;
     }
 }
 
 
-# encodes the 4 xml entities &amp; &lt; &gt; and &quot;.
-sub _xml_encode
-{
-    my $self = shift;
-    my $data = join '', @_;
-    return Petal::XML_Encode_Decode::encode ($data);
-}
-
-
 # this method fetches a Petal expression and returns it
 # without XML encoding. FETCH is basically a wrapper around
 # fetch() which looks for the special keyword 'structure'.
